@@ -1,3 +1,4 @@
+import hashlib
 import json
 import logging
 import time
@@ -24,34 +25,23 @@ class RemoteDevice:
     _CONFIG_REQ = {}
 
     def __init__(self, device_settings: dict, mqtt_settings: dict):
-        self._operation_topics = defaultdict(MQTTChannel)
 
-        self._config: dict = {}
-        self._config["object_id"] = device_settings["object_id"]
-        self._discovery_prefix = f"{mqtt_settings['discovery_prefix']}/" \
-                                 f"{device_settings['device_class']}/" \
-                                 f"{mqtt_settings['bridge_name']}/" \
-                                 f"{self.get_uid()}/"
+        uid_string = f"{mqtt_settings['operating_prefix']}_" \
+                     f"{mqtt_settings['bridge_name']}_" \
+                     f"{self.__class__.__name__}_" \
+                     f"{device_settings['object_id']}"
 
-        self._operating_prefix = f"{mqtt_settings['operating_prefix']}/" \
-                                 f"{mqtt_settings['bridge_name']}/" \
-                                 f"{self.get_uid()}/"
-
-        self._logging_channel = None
-        if mqtt_settings["logging"]:
-            self._logging_channel = f"{mqtt_settings['operating_prefix']}/" \
-                                    f"{mqtt_settings['bridge_name']}/" \
-                                    f"{self.get_uid()}/" \
-                                    f"log"
-
-        self._config["topic"] = f"{self._discovery_prefix}config"
-
-        self._add_channel(channel_name="state_topic",
-                          sub_topic="state",
-                          on_message=None)
+        uid_hash = hashlib.new("sha1", data=uid_string.encode(), usedforsecurity=False)
+        self._uid = uid_hash.hexdigest()[:16]
 
         self._device_class = device_settings['device_class']
-        self._config["name"] = device_settings["name"]
+
+        # prepare auto config dict
+        self._config: dict = {"device": {"identifiers": [f"{mqtt_settings['operating_prefix']}_"
+                                                         f"{mqtt_settings['bridge_name']}"],
+                                         "name": mqtt_settings['bridge_name']}, "name": device_settings["name"],
+                              "unique_id": self._uid,
+                              "object_id": device_settings["object_id"]}
 
         # some devices may need special attributes to appear in a special manner in hassio,
         # there are too many to handle all of them, so add them via generic dict from config on demand
@@ -66,6 +56,31 @@ class RemoteDevice:
             except AssertionError:
                 logging.warning(f"Could not overwrite a required item with the optional dict ({attribute})")
 
+        self._discovery_prefix = f"{mqtt_settings['discovery_prefix']}/" \
+                                 f"{device_settings['device_class']}/" \
+                                 f"{mqtt_settings['bridge_name']}/" \
+                                 f"{self.get_id()}/"
+
+        # store the discovery topic
+        self._config["discovery_topic"] = f"{self._discovery_prefix}config"
+
+        # prepare mqtt channels
+        self._operation_topics = defaultdict(MQTTChannel)
+        self._operating_prefix = f"{mqtt_settings['operating_prefix']}/" \
+                                 f"{mqtt_settings['bridge_name']}/" \
+                                 f"{self.get_id()}/"
+
+        self._logging_channel = None
+        if mqtt_settings["logging"]:
+            self._logging_channel = f"{mqtt_settings['operating_prefix']}/" \
+                                    f"{mqtt_settings['bridge_name']}/" \
+                                    f"{self.get_id()}/" \
+                                    f"log"
+
+        self._add_channel(channel_name="state_topic",
+                          sub_topic="state",
+                          on_message=None)
+
         # connect self._publish to the function publish
         self._publish = mqtt_settings["f_publish"]
 
@@ -78,15 +93,17 @@ class RemoteDevice:
             self._publish(topic=self._logging_channel,
                           payload=print_string,
                           retain=False,
-                          qos=1)
+                          qos=0)
 
-    def _update(self, channel_name: str, message: any) -> None:
+    def _update(self, channel_name: str, message: any, retain: bool = False, qos: int = 0) -> None:
         """
         Publish the message in json format to the channel name
         (must be added by add_channel first)
 
         :param channel_name: name of the device channel
         :param message: any type of json dumpable data to publish
+        :param retain: Retain flag for this message
+        :param qos: qos level for this message
         :return: None
         """
 
@@ -97,8 +114,8 @@ class RemoteDevice:
 
         self._publish(topic=topic,
                       payload=message,
-                      retain=False,
-                      qos=1)
+                      retain=retain,
+                      qos=qos)
 
     def _add_channel(self, channel_name: str, sub_topic: str, on_message=None) -> None:
         """
@@ -113,12 +130,21 @@ class RemoteDevice:
 
         self._operation_topics[channel_name] = MQTTChannel(self._operating_prefix + sub_topic, on_message)
 
-    def get_uid(self) -> str:
-        """Get a ID that is unique in this bridge
+    def get_id(self) -> str:
+        """Get a human readable ID of the device,
+        unique in this bridge
 
-        :return: Unique ID as string
+        :return: ID as string
         """
-        return f"{self.__class__.__name__}_{self._config['object_id']}"
+        return f"{self.__class__.__name__}_{self.get_object_id()}"
+
+    def get_uid(self) -> str:
+        """Get a real UID of the endpoint,
+        sha1 hash over bridge and device information
+
+        :return: UID as string
+        """
+        return self._uid
 
     def get_object_id(self) -> str:
         """Get the object id provided in the config file,
@@ -138,11 +164,11 @@ class RemoteDevice:
         """"
         Generate the config dictionary in MQTT Discovery stile
 
-        :return: auto discover tuple with the discovery channel on index 0
+        :return: auto discover tuple with the discovery topic on index 0
         """
 
         auto_config = {**self._config, **{name: channel.topic for (name, channel) in self._operation_topics.items()}}
-        topic = auto_config.pop("topic")
+        topic = auto_config.pop("discovery_topic")
 
         return topic, auto_config
 
@@ -160,7 +186,9 @@ class RemoteDevice:
         """Returns a dict with the required keys and the expected data types as values
         :return: dict
         """
-        return cls._CONFIG_REQ | cls._BASE_CONFIG_REQ
+        config_req = cls._CONFIG_REQ.copy()
+        config_req.update(cls._BASE_CONFIG_REQ)
+        return config_req
 
 
 def supported_device_classes() -> dict:
